<?php

namespace backend\modules\reports\controllers;

use Mpdf\Mpdf;
use Yii;
use yii\helpers\ArrayHelper;
use backend\modules\reports\models\CashFlowReport;
use backend\modules\reports\models\CashFlowReportSearch;
use backend\modules\users\models\PurchasePayment;
use backend\modules\users\models\ReceiptDetails;
use yii\data\ActiveDataProvider;
use backend\modules\master\models\Vendor;
use backend\modules\master\models\Customer;
use PHPExcel;
use PHPExcel_IOFactory;
class CashFlowController extends \yii\web\Controller
{
    public function actionIndex($print = null, $excel = null, $pdf = null)
    {
        $searchModel = new CashFlowReportSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        if ($print != null) {
            $dataProvider->pagination = false;
            $mpdf = new Mpdf();
            $mpdf->showImageErrors = true;
            $mpdf->WriteHTML($this->renderPartial('/cash-flow/export-pdf', [
                'dataProvider' => $dataProvider,
                'model' => $searchModel,
            ]));
            $mpdf->Output('Cash_flow_Report.pdf', 'I');
        }

        if ($pdf != null) {
            $dataProvider->pagination = false;
            $mpdf = new Mpdf();
            $mpdf->showImageErrors = true;
            $mpdf->WriteHTML($this->renderPartial('/cash-flow/export-pdf', [
                'dataProvider' => $dataProvider,
                'model' => $searchModel,
            ]));
            $mpdf->Output('Cash_flow_Report.pdf', 'D');
        }

        if ($excel != null) {
            $this->actionExcelReport($dataProvider);
        }

        return $this->render('index', [
            'dataProvider' => $dataProvider,
            'model' => $searchModel,
        ]);
    }

    public function actionDetailedTransaction($first_day, $last_day, $print = null, $pdf = null, $excel = null)
    {
        $searchModel = new CashFlowReportSearch();
        $searchModel->from_date = $first_day;
        $searchModel->to_date = $last_day;

        $query1 = ReceiptDetails::find()->where(['between', 'receipt_date', $searchModel->from_date, $searchModel->to_date]);
        $query2 = PurchasePayment::find()->where(['between', 'date', $first_day, $last_day])
                                         ->andWhere(['=', 'is_deleted', 0]);

        $dataProvider1 = new ActiveDataProvider([
            'query' => $query1,
            'pagination' => ['pageSize' => 20],
        ]);
        $dataProvider2 = new ActiveDataProvider([
            'query' => $query2,
            'pagination' => ['pageSize' => 20],
        ]);

        if ($print != null) {
            $dataProvider1->pagination = false;
            $dataProvider2->pagination = false;
            $mpdf = new Mpdf();
            $mpdf->showImageErrors = true;
            $mpdf->WriteHTML($this->renderPartial('/cash-flow/detail-export', [
                'dataProvider1' => $dataProvider1,
                'dataProvider2' => $dataProvider2,
                'model' => $searchModel,
            ]));
            $mpdf->Output('Cash_flow_Report.pdf', 'I');
        }

        if ($pdf != null) {
            $dataProvider1->pagination = false;
            $dataProvider2->pagination = false;
            $mpdf = new Mpdf();
            $mpdf->showImageErrors = true;
            $mpdf->WriteHTML($this->renderPartial('/cash-flow/detail-export', [
                'dataProvider1' => $dataProvider1,
                'dataProvider2' => $dataProvider2,
                'model' => $searchModel,
            ]));
            $mpdf->Output('Cash_flow_Report.pdf', 'D');
        }

        if ($excel != null) {
            $this->actionDetailExcelReport($dataProvider1, $dataProvider2);
        }

        return $this->render('detailed-transaction', [
            'dataProvider1' => $dataProvider1,
            'dataProvider2' => $dataProvider2,
            'model' => $searchModel,
        ]);
    }

    public function actionExportPrint()
    {
        $searchModel = new CashFlowReportSearch();
        $now = new \DateTime('now');
        $month = $now->format('m');
        $year = $now->format('Y');
        $searchModel->month_year = $year . '-' . $month;

        $dataProvider = $searchModel->search([]);
        $dataProvider->pagination = false;

        $mpdf = new Mpdf();
        $mpdf->showImageErrors = true;
        $mpdf->WriteHTML($this->renderPartial('/cash-flow/export-pdf', [
            'dataProvider' => $dataProvider,
            'model' => $searchModel,
        ]));
        $mpdf->Output('Cash_flow_Report.pdf', 'I');
    }

    public function actionExportPdf()
    {
        $searchModel = new CashFlowReportSearch();
        $now = new \DateTime('now');
        $month = $now->format('m');
        $year = $now->format('Y');
        $searchModel->month_year = $year . '-' . $month;

        $dataProvider = $searchModel->search([]);
        $dataProvider->pagination = false;

        $mpdf = new Mpdf();
        $mpdf->showImageErrors = true;
        $mpdf->WriteHTML($this->renderPartial('/cash-flow/export-pdf', [
            'dataProvider' => $dataProvider,
            'model' => $searchModel,
        ]));
        $mpdf->Output('Cash_flow_Report.pdf', 'D');
    }
    public function indianCurrencyFormat($number) {
        $decimal = '';
        if (strpos($number, '.') !== false) {
            list($number, $decimal) = explode('.', $number);
            $decimal = '.' . substr($decimal, 0, 2); // Limit to 2 decimal places
        }
    
        // Format the number before the decimal point
        $lastThree = substr($number, -3);
        $restUnits = substr($number, 0, -3);
        if (strlen($restUnits) > 0) {
            $restUnits = preg_replace('/\B(?=(\d{2})+(?!\d))/', ',', $restUnits);
            $formattedNumber = $restUnits . ',' . $lastThree;
        } else {
            $formattedNumber = $lastThree;
        }
    
        return $formattedNumber . $decimal;
    }
    public function indianFormat($number) {
        // Ensure the number is treated as an integer for formatting purposes
        $number = intval($number);
    
        // Format the number
        $lastThree = substr($number, -3);
        $restUnits = substr($number, 0, -3);
    
        if (strlen($restUnits) > 0) {
            $restUnits = preg_replace('/\B(?=(\d{2})+(?!\d))/', ',', $restUnits);
            $formattedNumber = $restUnits . ',' . $lastThree;
        } else {
            $formattedNumber = $lastThree;
        }
    
        return $formattedNumber;
    }
    
    
    
    public function actionExcelReport($data = null)
    {
        if ($data !== null) {
            $dataProvider = $data;
        } else {
            $query = CashFlowReport::find();
            $now = new \DateTime('now');
            $month = $now->format('m');
            $year = $now->format('Y');
            $query->andFilterWhere(['month_year' => $year . '-' . $month]);
            $dataProvider = new ActiveDataProvider([
                'query' => $query,
                'pagination' => false,
            ]);
        }
    
        // Create a new PHPSpreadsheet instance
        $spreadsheet = new \PhpOffice\PhpSpreadsheet\Spreadsheet();
        $sheet = $spreadsheet->getActiveSheet();
    
        // Add data to the sheet
        $sheet->setCellValue('A1', 'MOHAPATRA SCIENTIFIC PRIVATE LIMITED');
        $sheet->setCellValue('A4', 'F2G/30, INDRADHANU MARKET, IRC VILLAGE,');
        $sheet->setCellValue('A5', 'BHUBANESWAR, Odisha');
        $sheet->setCellValue('A7', 'Cash Flow Report');
        $sheet->setCellValue('A8', '');
    
        $sheet->getStyle('A1:D1')->getFont()->setBold(true)->setSize(11);
        $sheet->getStyle('A2:D2')->getFont()->setSize(11);
        $sheet->getStyle('A3:D3')->getFont()->setSize(11);
        $sheet->getStyle('A7:D7')->getFont()->setSize(11);
        $sheet->getStyle('A5:D5')->getFont()->setSize(11);
        $sheet->getStyle('A8:D8')->getFont()->setSize(11);
    
        $sheet->getColumnDimension('A')->setWidth(10);
        $sheet->getColumnDimension('B')->setWidth(10);
        $sheet->getColumnDimension('C')->setWidth(10);
        $sheet->getColumnDimension('D')->setWidth(10);
    
        $sheet->mergeCells('A1:D1');
        $sheet->mergeCells('A2:D2');
        $sheet->mergeCells('A3:D3');
        $sheet->mergeCells('A4:D4');
        $sheet->mergeCells('A5:D5');
        $sheet->mergeCells('A6:D6');
        $sheet->mergeCells('A7:D7');
        $sheet->mergeCells('A8:D8');
        $sheet->mergeCells('B9:C9');
    
        $sheet->setCellValue('B9', 'Cash Movement');
        $sheet->setCellValue('A10', 'Particulars');
        $sheet->setCellValue('B10', 'Inflow');
        $sheet->setCellValue('C10', 'Outflow');
        $sheet->setCellValue('D10', 'Net Flow');
    
        $row = 11;
        $total_outflow = 0;
        $total_inflow = 0;
        foreach ($dataProvider->getModels() as $model) {
            $total_inflow += $model->inflow;
            $total_outflow += $model->outflow;
        }
    
        foreach ($dataProvider->getModels() as $model) {
            $sheet->setCellValue('A' . $row, date('F-Y', strtotime($model->month_year)));
            $sheet->setCellValue('B' . $row, $model->inflow);
            $sheet->setCellValue('C' . $row, $model->outflow);
            $sheet->setCellValue('D' . $row, $model->net_flow);
            $row++;
        }
    
        $sheet->setCellValue('A' . $row, 'Grand Total');
        $sheet->setCellValue('B' . $row, $total_inflow);
        $sheet->setCellValue('C' . $row, $total_outflow);
        $sheet->setCellValue('D' . $row, $total_inflow - $total_outflow);
    
        $writer = new \PhpOffice\PhpSpreadsheet\Writer\Xlsx($spreadsheet);
    
        // Send the Excel file to the browser
        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment; filename="cash-flow.xlsx"');
        $writer->save('php://output');
        exit();
    }    
    public function actionDetailExcelReport($data1 = null, $data2 = null)
    {
        if ($data1 !== null && $data2 !== null) {
            $dataProvider1 = $data1;
            $dataProvider2 = $data2;
        }

        // Create a new PHPSpreadsheet instance
        $spreadsheet = new \PhpOffice\PhpSpreadsheet\Spreadsheet();
        $sheet = $spreadsheet->getActiveSheet();

        // Add data to the sheet
        $sheet->setCellValue('A1', 'MOHAPATRA SCIENTIFIC PRIVATE LIMITED');
        $sheet->setCellValue('A4', 'F2G/30, INDRADHANU MARKET, IRC VILLAGE,');
        $sheet->setCellValue('A5', 'BHUBANESWAR, Odisha');
        $sheet->setCellValue('A7', 'Cash Flow Report');
        $sheet->setCellValue('A8', '');

        $sheet->getStyle('A8:C8')->getFont()->setBold(true)->setSize(11);
        $sheet->getStyle('A2:D2')->getFont()->setSize(11);
        $sheet->getStyle('A3:D3')->getFont()->setSize(11);
        $sheet->getStyle('A7:D7')->getFont()->setSize(11);
        $sheet->getStyle('A5:D5')->getFont()->setSize(11);
        $sheet->getStyle('A8:D8')->getFont()->setSize(11);

        $sheet->getColumnDimension('A')->setWidth(10);
        $sheet->getColumnDimension('B')->setWidth(10);
        $sheet->getColumnDimension('C')->setWidth(10);
        $sheet->getColumnDimension('D')->setWidth(10);

        $sheet->mergeCells('A8:B8');
        $sheet->mergeCells('C8:D8');
        $sheet->mergeCells('A1:D1');
        $sheet->mergeCells('A4:D4');
        $sheet->mergeCells('A5:D5');
        $sheet->mergeCells('A6:D6');
        $sheet->mergeCells('A7:D7');

        $sheet->setCellValue('A8', 'Inflow');
        $sheet->setCellValue('C8', 'Outflow');

        $row1 = 9;
        $total_inflow = 0;
        $total_outflow = 0;
        foreach ($dataProvider1->getModels() as $model1) {
            $total_inflow += $model1->total_amount;
        }
        foreach ($dataProvider2->getModels() as $model2) {
            $total_outflow += $model2->total_amount;
        }

        foreach ($dataProvider1->getModels() as $model1) {
            $customer = Customer::find()->where(['id' => $model1->customer_name])->one();
            $sheet->setCellValue('A' . $row1, $customer ? $customer->business_name : $model1->customer_name);
            $sheet->setCellValue('B' . $row1, $model1->total_amount);
            $row1++;
        }

        $row2 = 9;
        foreach ($dataProvider2->getModels() as $model2) {
            $vendor = Vendor::find()->where(['id' => $model2->vendor_id])->one();
            $sheet->setCellValue('C' . $row2, $vendor ? $vendor->business_name : 'Unknown Vendor');
            $sheet->setCellValue('D' . $row2, $model2->total_amount);
            $row2++;
        }

        $row = max($row1, $row2);
        $sheet->getStyle('A' . ($row + 1) . ':D' . ($row + 1))->getFont()->setBold(true)->setSize(11);
        $sheet->mergeCells('A' . ($row + 1) . ':C' . ($row + 1));

        $sheet->setCellValue('A' . $row, 'Inflow Total');
        $sheet->setCellValue('B' . $row, $total_inflow);
        $sheet->setCellValue('C' . $row, 'Outflow Total');
        $sheet->setCellValue('D' . $row, $total_outflow);
        $sheet->setCellValue('A' . ($row + 1), 'Netflow');
        $sheet->setCellValue('D' . ($row + 1), $total_inflow - $total_outflow);

        $writer = new \PhpOffice\PhpSpreadsheet\Writer\Xlsx($spreadsheet);

        // Send the Excel file to the browser
        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment; filename="cash-flow-details.xlsx"');
        $writer->save('php://output');
        exit();
    }

}
